﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Npgsql;
using Nyilvantarto.mdl;

namespace Nyilvantarto
{
    /// <summary>
    /// A program fő ablakának eseményeit kezelő osztály.
    /// </summary>
    public partial class Form1 : Form
    {
        private DataConnection kapcsolat;
        private DataSet ds = new DataSet();
        private DataTable dt = new DataTable();
        private Boolean jarmuListaVan = false;

        /// <summary>
        /// Kapcsolatleíró sztring.
        /// </summary>
        public string connstring;
        private bool ingatlanListaVan = false;
       

        public Form1()
        {
            InitializeComponent();
            kapcsolat = new DataConnection();
            
        }


        private void szerkesztéaToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void névjegyToolStripMenuItem_Click(object sender, EventArgs e)
        {

            MessageBox.Show("Nyilvántartó rendszer\nMintaalkalmazás a PostgreSQL objektum-relációs lehetőségeinek bemutatására\n(C) Cser Lajos, 2012-2013." );
        }

        private void kilépésToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void kapcsolatNyitásaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (kapcsolat.connect())
            {
                toolStripStatusLabel1.Text = "Kapcsolódva";
            }
        }

        private void kapcsolatBontásaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (kapcsolat.disconnect())
            {
                toolStripStatusLabel1.Text = "Kapcsolat bontva";
            }
        }

        private void dolgozókListázásaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adatLista.DataSource = kapcsolat.listDolgozok();
            jarmuListaVan = false;
            ingatlanListaVan = false;
        }

        private void újDolgozóFelvételeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UjDolgozo dolg = new UjDolgozo(kapcsolat);
            dolg.Show();
        }

        private void szerverKonfigurációToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form2 konfig = new Form2(kapcsolat);
            konfig.ShowDialog();
        }

        private void keresésToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Kereses frm = new Kereses(kapcsolat, this);
            frm.Show();
        }

        private void járművekÖsszÁtlagfogyasztásaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adatLista.DataSource = kapcsolat.list("SELECT SUM(j.OsszFogyasztas) osszesitett_fogyasztas FROM Jarmu j;");
            jarmuListaVan = false;
            ingatlanListaVan = false;

        }

        private void dolgozókTörléseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!jarmuListaVan && !ingatlanListaVan)
            {
                DialogResult dialogResult = MessageBox.Show("Biztosan törli a kijelölt dolgozókat?", "Megerősítés", MessageBoxButtons.YesNo);
                if (dialogResult == DialogResult.Yes)
                {
                    foreach (DataGridViewRow sor in adatLista.SelectedRows)
                    {
                        kapcsolat.deleteDolgozo(sor.Cells[1].Value.ToString());
                    }
                }
            }
            else
            {
                MessageBox.Show("Ez a művelet csak akkor hajtható végre, ha jelenleg dolgozók vannak listázva.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        private void dolgozókLétszámaToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adatLista.DataSource = kapcsolat.list("SELECT COUNT(*) dolgozo_letszam FROM dolgozo;");
            jarmuListaVan = false;
            ingatlanListaVan = false;
        }

        private void toolStripDropDownButton1_Click(object sender, EventArgs e)
        {
            adatLista.DataSource = kapcsolat.listJarmuvek();
            jarmuListaVan = true;
            ingatlanListaVan = false;
        }

        private void újJárműFelvételeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UjJarmu jrm = new UjJarmu(kapcsolat);
            jrm.ShowDialog();
        }

        private void járműTankolásainakKezeléseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (jarmuListaVan)
            {
                List<TankolasKezelo> ablakok = new List<TankolasKezelo>();
                foreach (DataGridViewRow sor in adatLista.SelectedRows)
                {
                    ablakok.Add(new TankolasKezelo(kapcsolat, sor.Cells[0].Value.ToString()));
                }
                foreach (Form f in ablakok)
                {
                    f.Show();
                }
            }
            else
            {
                MessageBox.Show("Ez a művelet csak akkor hajtható végre, ha jelenleg járművek vannak listázva.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        private void témakörökToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Sugo s = new Sugo();

            s.Show();
        }

        private void személyautókKezeléseToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void járművekMennyiségeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adatLista.DataSource = kapcsolat.list("SELECT * FROM jarmu_count();");
            jarmuListaVan = false;
            ingatlanListaVan = false;
        }

        private void munkagépekToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        public void setAdatListaDataSource(DataTable t)
        {
            jarmuListaVan = false;
            ingatlanListaVan = false;
            this.adatLista.DataSource = t;
        }

        private void toolStripSplitButton3_ButtonClick(object sender, EventArgs e)
        {
            jarmuListaVan = false;
            ingatlanListaVan = true;
            this.adatLista.DataSource = kapcsolat.findIngatlan(new utils.Criteria());
        }

        private void újFelvételeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UjIngatlan frm = new UjIngatlan(kapcsolat);
            frm.Show();
        }

        private void dolgozókÁtlagéletkoraToolStripMenuItem_Click(object sender, EventArgs e)
        {
            adatLista.DataSource = kapcsolat.list("SELECT AVG(ABS((EXTRACT(YEAR FROM szuletesidatum))-(EXTRACT(YEAR FROM current_date)))) atlageletkor FROM dolgozo;");
            jarmuListaVan = false;
            ingatlanListaVan = false;       
        }

        private void járművekTörléseToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (jarmuListaVan)
            {
                DialogResult dialogResult = MessageBox.Show("Biztosan törli a kijelölt járműveket?", "Megerősítés", MessageBoxButtons.YesNo);
                if (dialogResult == DialogResult.Yes)
                {
                    foreach (DataGridViewRow sor in adatLista.SelectedRows)
                    {
                        kapcsolat.deleteJarmu(sor.Cells[0].Value.ToString());
                    }
                }
            }
            else
            {
                MessageBox.Show("Ez a művelet csak akkor hajtható végre, ha jelenleg járművek vannak listázva.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        private void jelentésekToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                string s = "";
                int dolgozoletszam = int.Parse(kapcsolat.list("SELECT COUNT(*) dolgozo_letszam FROM dolgozo;").Rows[0][0].ToString());
                double dolgozoelet = double.Parse(kapcsolat.list("SELECT AVG(ABS((EXTRACT(YEAR FROM szuletesidatum))-(EXTRACT(YEAR FROM current_date)))) atlageletkor FROM dolgozo;").Rows[0][0].ToString());
                double osszfogy = double.Parse(kapcsolat.list("SELECT SUM(j.OsszFogyasztas) osszesitett_fogyasztas FROM Jarmu j;").Rows[0][0].ToString());
                int osszjarmu = int.Parse(kapcsolat.list("SELECT COUNT(*) FROM Jarmu;").Rows[0][0].ToString());
                int teherauto = int.Parse(kapcsolat.list("SELECT COUNT(*) FROM Teherauto;").Rows[0][0].ToString());
                int munkagep = int.Parse(kapcsolat.list("SELECT COUNT(*) FROM Munkagep;").Rows[0][0].ToString());
                int szemelyauto = int.Parse(kapcsolat.list("SELECT COUNT(*) FROM Szemelyauto;").Rows[0][0].ToString());
                int egyeb = int.Parse(kapcsolat.list("SELECT COUNT(*) FROM ONLY Jarmu;").Rows[0][0].ToString());
                Jelentes j = new Jelentes();
                j.setText("Nyilvántartó" + Environment.NewLine + "Mintaalkalmazás a PostgreSQL objektum-relációs lehetőségei c. szakdolgozathoz"
                    + Environment.NewLine + "-----------------------------------------------------------------------------"
                    + Environment.NewLine + Environment.NewLine + "JELENTÉS a rendszer főbb paramétereivel"
                    + Environment.NewLine + Environment.NewLine +
                    "1. Dolgozók"
                    + Environment.NewLine + Environment.NewLine +
                    "1.1. Dolgozók létszáma: " + dolgozoletszam + Environment.NewLine +
                    "1.2. Dolgozók átlagéletkora: " + dolgozoelet + " év"
                    + Environment.NewLine + Environment.NewLine +
                    "2. Járműállomány"
                    + Environment.NewLine + Environment.NewLine +
                    "2.1. Összegzett fogyasztás: " + osszfogy + " liter (benzin és diesel)"
                    + Environment.NewLine + 
                    "2.2. Járművek mennyisége" + Environment.NewLine +
                    "  - Összesen: " + osszjarmu + Environment.NewLine +
                    "  - Személyautó: " + szemelyauto + Environment.NewLine +
                    "  - Teherautó: " + teherauto + Environment.NewLine +
                    "  - Munkagép: " + munkagep + Environment.NewLine +
                    "  - Egyéb jármű: " + egyeb + Environment.NewLine 
                    );
                j.Show();
            }
            catch (Exception err)
            {
                MessageBox.Show("Hiba a jelentés összeállítása során. Ellenőrizze a kapcsolat állapotát.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

    }
}
