﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Windows.Forms;

using System.Data;
using Npgsql;
using NpgsqlTypes;

using Nyilvantarto.mdl.ent;
using Nyilvantarto.mdl.ent.types;
using Nyilvantarto.utils;

namespace Nyilvantarto.mdl
{
    /// <summary>
    /// Az adatbázis kezelésével kapcsolatos alapvető tennivalókat megvalósító osztály
    /// (létrehozás, olvasás, frissítés, törlés.)
    /// </summary>
    public class DataConnection
    {
        private DataSet ds = new DataSet();
        private DataTable dt = new DataTable();
        private NpgsqlConnection conn;
        private Boolean connState = false;

        /// <summary>
        /// Kapcsolatleíró sztring.
        /// </summary>
        public string ConnectionString
        {
            get;
            set;
        }

        /// <summary>
        /// Új adatkapcsolat lérehozásáért felelős konstruktor. Inicializálja a kapcsolatot
        /// az alapértelmezett adatokkal, de nem nyitja meg azt.
        /// </summary>
        public DataConnection()
        {
            conn = new NpgsqlConnection();
            ConnectionString = String.Format("Server={0};Port={1};" +
                      "User Id={2};Password={3};Database={4};",
                      "localhost", "5432", "postgres",
                      "aaa", "teszt");
        }

        /// <summary>
        /// Kapcsolat megnyitása (inicializáció után.)
        /// </summary>
        /// <returns>A kapcsolat sikeres megnyitása esetén igazzal tér vissza, egyébként hamissal.</returns>
        public Boolean connect()
        {
            if (connState == false)
            {
                conn = new NpgsqlConnection(ConnectionString);
                try
                {
                    conn.Open();
                }
                catch (NpgsqlException error)
                {
                    MessageBox.Show("Hiba lépett fel a csatlakozás során. Ellenőrizze a konfiguráció helyességét. A hibaüzenet:\n\n" + error.Message, "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return false;
                }
                if (conn.State == System.Data.ConnectionState.Open)
                {
                    connState = true;
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Kapcsolat bontása
        /// </summary>
        /// <returns>Igaz, ha a kapcsolat bontása sikeres volt</returns>
        public Boolean disconnect()
        {
            if (connState == true)
            {
                try
                {
                    conn.Close();
                    connState = false;
                    return true;
                }
                catch (NpgsqlException error)
                {
                    MessageBox.Show("Hiba lépett fel a csatlakozás során. Ellenőrizze a konfiguráció helyességét. A hibaüzenet:\n\n" + error.Message, "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return false;
                }
            }
            else
            {
                MessageBox.Show("Hiba lépett fel a bontás során - a kapcsolat jelenleg nem nyitott.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return false;
            }
        }

        /// <summary>
        /// Egyéni SQL-lekérdezés végrehajtására szolgáló metódus.
        /// </summary>
        /// <param name="sql">Az SQL utasítás szövegként</param>
        /// <returns>A visszaadott adatok halmaza</returns>
        public NpgsqlDataReader dbExecute(string sql)
        {
            if (conn.State == System.Data.ConnectionState.Open)
            {
                NpgsqlCommand cmd = new NpgsqlCommand(sql, conn);
                NpgsqlDataReader r = cmd.ExecuteReader();
                return r;
            }
            else
            {
                MessageBox.Show("Nincs élő kapcsolat!", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
            return null;
        }

        /// <summary>
        /// Új dolgozó beszúrása.
        /// </summary>
        /// <param name="Uj">Egy dolgozó egyed, ami tartalmazza az adatokat.</param>
        public void insertDolgozo(Dolgozo Uj)
        {
            if (conn.State == System.Data.ConnectionState.Open)
            {
                try
                {
                    string statement;
                    statement = String.Format(@"INSERT INTO Dolgozo VALUES (
	                                            '{1}',
	                                            '{0}',
	                                            '{2}',
	                                            '{3}',
	                                            ROW(
                                                   '{4}',
		                                            {5},
		                                            {6}
	                                            )
                                            )", Uj.Nev, Uj.SzemelyiSzam, Uj.SzuletesiDatum, Uj.Beosztas,
                                              Uj.Szerzodes, Uj.Probaidos ? "TRUE" : "FALSE", Uj.ProbaidoHossz);
                    NpgsqlCommand stmt = conn.CreateCommand();
                    stmt.CommandText = statement;
                    stmt.ExecuteNonQuery();
                }
                catch (NpgsqlException err)
                {
                    MessageBox.Show("Hiba történt az adatbázis-művelet során. A hiba részletezése:\n\n"+err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            else
            {
                MessageBox.Show("Nincs élő kapcsolat!", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public DataTable list(String sql)
        {
            if (conn.State == System.Data.ConnectionState.Open)
            {
                try
                {
                    NpgsqlDataAdapter da = new NpgsqlDataAdapter(sql, conn);
                    ds.Reset();
                    da.Fill(ds);
                    dt = ds.Tables[0];
                    return dt.Copy();
                }
                catch (NpgsqlException err)
                {
                    MessageBox.Show("Hiba történt a végrehajtás során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return null;
                }
            }
            else
            {
                MessageBox.Show("Nincs élő kapcsolat!", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return null;
            }
        }

        private void insert(String sql)
        {
            if (conn.State == System.Data.ConnectionState.Open)
            {
                try
                {
                    NpgsqlCommand stmt = conn.CreateCommand();
                    stmt.CommandText = sql;
                    stmt.ExecuteNonQuery();
                }
                catch (NpgsqlException err)
                {
                    MessageBox.Show("Hiba történt az adatbázis-művelet során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            else
            {
                MessageBox.Show("Nincs élő kapcsolat!", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        /// <summary>
        /// Dolgozók listázása.
        /// </summary>
        /// <returns>A dolgozókat tartalmazó adattábla</returns>
        public DataTable listDolgozok()
        {
            return list(@"SELECT Nev, SzemelyiSzam, SzuletesiDatum, Beosztas, 
                                CASE 
	                                WHEN(Szerzodese).Probaido = FALSE THEN 'nem' 
	                                ELSE 'igen'
                                END AS Probaidos 
                                FROM Dolgozo;");
        }

        /// <summary>
        /// Előállítja a járművek összesített listáját.
        /// </summary>
        /// <returns>A járműveket tartalmazó tábla</returns>
        public DataTable listJarmuvek()
        {
            return list("SELECT alvazszam, uzemanyag, gyarto, tipus FROM Jarmu");
        }

        /// <summary>
        /// Új jármű beszúrása.
        /// </summary>
        /// <param name="uj">A beszúrandó jármű</param>
        public void insertJarmu(Jarmu uj)
        {
            try
            {
                String sql;
                if (uj is Szemelyauto)
                {
                    sql = String.Format(@"INSERT INTO Szemelyauto VALUES ('{0}', '{1}', '{2}', '{3}', '{{}}', {4}, {5})",
                        uj.Alvazszam, uj.Uzemanyag, uj.Gyarto, uj.Tipus,
                        uj.UtasMax.ToString(), uj.CsomagtartoMeret.ToString());
                    insert(sql);
                }
                else if (uj is Teherauto)
                {
                    sql = String.Format(@"INSERT INTO Teherauto VALUES ('{0}', '{1}', '{2}', '{3}', '{{}}', {4})",
                       uj.Alvazszam, uj.Uzemanyag, uj.Gyarto, uj.Tipus,
                       uj.Kapacitas);
                    insert(sql);
                }
                else if (uj is Munkagep)
                {
                    sql = String.Format(@"INSERT INTO Munkagep VALUES ('{0}', '{1}', '{2}', '{3}', '{{}}', '{4}', {5})",
                        uj.Alvazszam, uj.Uzemanyag, uj.Gyarto, uj.Tipus,
                        uj.fajta, uj.motorero);
                    insert(sql);
                }
                else
                {
                    MessageBox.Show("Nem meghatározott objektumtípus.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception err)
            {
                MessageBox.Show("Hiba történt a végrehajtás során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

        }

        public DataTable listTankolas(string vin)
        {
            return list(String.Format("select (unnest(tankolasok)).datum, (unnest(tankolasok)).bizonylatszam, (unnest(tankolasok)).liter from jarmu where alvazszam = '{0}'", vin));
        }

        public void saveTankolas(List<TankolasRekord> t, string vin)
        {
            foreach (TankolasRekord elem in t)
            {
                if (conn.State == System.Data.ConnectionState.Open)
                {
                    try
                    {
                        string statement;
                        statement = String.Format(@"UPDATE jarmu 
                                                    SET Tankolasok[Jarmu.UjTankolas] = 
                                                    ROW('{0}', '{1}', {2}) WHERE
                                                    Alvazszam = '{3}';", elem.Datum, elem.Bizonylatszam, elem.Liter.ToString().Replace(',','.'), vin);
                        NpgsqlCommand stmt = conn.CreateCommand();
                        stmt.CommandText = statement;
                        stmt.ExecuteNonQuery();
                    }
                    catch (NpgsqlException err)
                    {
                        MessageBox.Show("Hiba történt az adatbázis-művelet során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                }
                else
                {
                    MessageBox.Show("Nincs élő kapcsolat!", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
        }


        public DataTable findJarmu(Criteria feltetel)
        {
            return list("select * from jarmu " + feltetel.getCriteria());
        }

        public DataTable findDolgozo(Criteria feltetel)
        {
            return list("select * from dolgozo " + feltetel.getCriteria());
        }

        public DataTable findIngatlan(Criteria feltetel)
        {
            return list("select * from ingatlan " + feltetel.getCriteria());
        }

        public void insertIngatlan(Ingatlan uj)
        {
            try
            {
                String sql;
                if (uj is Iroda)
                {
                    sql = String.Format(@"INSERT INTO Iroda VALUES ('{0}', ROW({1}, {2}), {3}, ROW('{4}', '{5}', '{6}', '{7}', '{8}'), '{9}')",
                        uj.helyrajziszam, uj.tulajdonihanyad.szamlalo, uj.tulajdonihanyad.nevezo, uj.tulajdonszerzesev,
                        uj.postaicim.irsz, uj.postaicim.varos, uj.postaicim.utca, uj.postaicim.hazszam, uj.postaicim.orszag, uj.funkcio);
                    insert(sql);
                }
                else
                {
                    MessageBox.Show("Nem meghatározott objektumtípus.", "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception err)
            {
                MessageBox.Show("Hiba történt a végrehajtás során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public void deleteJarmu(string vin)
        {
            try
            {
                string statement;
                statement = String.Format(@"DELETE FROM Jarmu WHERE Alvazszam = '{0}'", vin);
                NpgsqlCommand stmt = conn.CreateCommand();
                stmt.CommandText = statement;
                stmt.ExecuteNonQuery();
                statement = String.Format(@"DELETE FROM Szemelyauto WHERE Alvazszam = '{0}'", vin);
                stmt.ExecuteNonQuery();
                statement = String.Format(@"DELETE FROM Munkagep WHERE Alvazszam = '{0}'", vin);
                stmt.ExecuteNonQuery();
                statement = String.Format(@"DELETE FROM Teherauto WHERE Alvazszam = '{0}'", vin);
                stmt.ExecuteNonQuery();
              
            }
            catch (Exception err)
            {
                MessageBox.Show("Hiba történt a végrehajtás során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }

        public void deleteDolgozo(string szemelyiszam)
        {
            try
            {
                string statement;
                statement = String.Format(@"DELETE FROM Dolgozo WHERE Szemelyiszam = '{0}'", szemelyiszam);
                NpgsqlCommand stmt = conn.CreateCommand();
                stmt.CommandText = statement;
                stmt.ExecuteNonQuery();

            }
            catch (Exception err)
            {
                MessageBox.Show("Hiba történt a végrehajtás során. A hiba részletezése:\n\n" + err.ToString(), "Hiba", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }
        }
    }
}
