﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Npgsql;
using NpgsqlTypes;

namespace Nyilvantarto.mdl.ent
{
    public class Dolgozo
    {
        /// <summary>
        /// A dolgozó neve.
        /// </summary>
        public String Nev { get; set; }
        /// <summary>
        /// A dolgozó személyi száma.
        /// </summary>
        public String SzemelyiSzam { get; set; }
        /// <summary>
        /// A dolgozó születésének dátuma.
        /// </summary>
        public DateTime SzuletesiDatum { get; set; }
        /// <summary>
        /// A dolgozó beosztása.
        /// </summary>
        public String Beosztas { get; set; }
        /// <summary>
        /// A dolgozó szerződésének részletezése.
        /// </summary>
        public String Szerzodes { get; set; }
        /// <summary>
        /// Igaz, ha a dolgozó jelenleg próbaidős.
        /// </summary>
        public Boolean Probaidos { get; set; }
        /// <summary>
        /// A dolgozó esetleges próbaidejének hossza.
        /// </summary>
        public int ProbaidoHossz { get; set; }

        /// <summary>
        /// Alapértelmezett konstruktor (üres adatokat hoz létre.)
        /// </summary>
        public Dolgozo()
        {
            this.Nev = "";
            this.SzemelyiSzam = "";
            this.Beosztas = "";
            this.Szerzodes = "";
            this.SzuletesiDatum = new DateTime(1900,01,01);
            this.Probaidos = false;
            this.ProbaidoHossz = 1;
        }

        /// <summary>
        /// Saját adatok megadásával létrehoz egy dolgozó egyedet.
        /// </summary>
        /// <param name="Nev">A dolgozó neve</param>
        /// <param name="SzemelyiSzam">A dolgozó személyi száma</param>
        /// <param name="SzuletesiDatum">A dolgozó születési dátuma</param>
        /// <param name="Beosztas">A dolgzozó beosztása</param>
        /// <param name="Szerzodes">A dolgozó szerződésének részletezése</param>
        /// <param name="Probaidos">Igaz, ha a dolgozó próbaidős</param>
        /// <param name="ProbaidoHossz">Az esetleges próbaidő hossza</param>
        public Dolgozo(String Nev, String SzemelyiSzam, DateTime SzuletesiDatum, String Beosztas,
            String Szerzodes, Boolean Probaidos, int ProbaidoHossz)
        {
            this.Nev = Nev;
            this.SzemelyiSzam = SzemelyiSzam;
            this.SzuletesiDatum = SzuletesiDatum;
            this.Beosztas = Beosztas;
            this.Szerzodes = Szerzodes;
            this.Probaidos = Probaidos;
            this.ProbaidoHossz = ProbaidoHossz;
        }
    }
}
