﻿/*
 * A PostgreSQL obejtum-relációs lehetőségei
 * Szakdolgozat
 * Cser Lajos
 * Szegedi Tudományegyetem, 2013.
 */

---------------------------------------------------------------
--            Típusdefiníciók, adatbázis létrehozása         --
---------------------------------------------------------------

-- Törtszám típus
CREATE TYPE Tort AS (
	Szamlalo INT,
	Nevezo INT
);

-- Postai cím típus
CREATE TYPE PostaiCim AS (
	Irsz CHARACTER VARYING,
	Varos CHARACTER VARYING,
	Utca CHARACTER VARYING,
	Hazszam CHARACTER VARYING,
	Orszag CHARACTER VARYING
);

-- A járművek tankolási adatait egy tömbben tároljuk, amely
-- ilyen rekordokból áll
CREATE TYPE TankolasRekord AS (
	Datum DATE,
	Bizonylatszam CHARACTER VARYING,
	Liter REAL
);

-- Hasonlóan a járművekhez, itt is egy nem-1NF megoldást
-- alkalmazunk 
CREATE TYPE HozamRekord AS (
	Ev INT,
	Termeny CHARACTER VARYING,
	Eredmeny REAL
);


CREATE TYPE TermenyTortenetRekord AS (
	Ev INT,
	Termeny CHARACTER VARYING,
	Eredmeny NUMERIC
);

-- Parcella rekord (értsd: egy adott kert egyes részei)
CREATE TYPE ParcellaRekord AS (
	Meret NUMERIC,
	Termeny CHARACTER VARYING,
	KorabbiTermenyek TermenyTortenetRekord[]
);

--------------------------------------------------------------
--------------------------------------------------------------

CREATE TABLE Ingatlan (
	HelyrajziSzam CHARACTER VARYING,
	TulajdoniHanyad Tort,
	TulajdonSzerzesEv INT,
	PRIMARY KEY (HelyrajziSzam)
);

CREATE TABLE Iroda (
	Cim PostaiCim,
	Funckio CHARACTER VARYING
) INHERITS (ingatlan);

CREATE TABLE Raktar (
	Alapterulet INT,
	Funkcio CHARACTER VARYING,
	Cim PostaiCim
) INHERITS (Ingatlan);

CREATE TABLE Foldterulet (
	Meret NUMERIC,
	ERTEK NUMERIC,
	CONSTRAINT foldterulet_pk PRIMARY KEY (HelyrajziSzam)
) INHERITS (Ingatlan);

CREATE TABLE Szolo (
	Szolofajta CHARACTER VARYING[],
	SorokSzama INT
) INHERITS (Foldterulet);

CREATE TABLE Kert (
	Parcellak ParcellaRekord[]
) INHERITS (Foldterulet);

CREATE TABLE Szanto (
	JelenlegiTermeny CHARACTER VARYING,
	MultbeliHozamok TermenyTortenetRekord[],
	CONSTRAINT szanto_pk PRIMARY KEY (HelyrajziSzam)
) INHERITS (Foldterulet);

CREATE TABLE Vegyszerezes (
	Azonosito INT,
	Datum DATE,
	Leiras CHARACTER VARYING,
	Helye CHARACTER VARYING,
	CONSTRAINT szanto_kulsk FOREIGN KEY (Helye) REFERENCES Szanto(HelyrajziSzam)
);

CREATE TABLE Gyumolcsos (
	Fajta CHARACTER VARYING,
	Egyedszam INT,
	TelepitesEve INT
) INHERITS (Foldterulet);

CREATE TABLE Jarmu (
	Alvazszam CHARACTER VARYING,
	Uzemanyag CHARACTER VARYING,
	Gyarto CHARACTER VARYING,
	Tipus CHARACTER VARYING,
	Tankolasok TankolasRekord[],
	CONSTRAINT jarmu_pk PRIMARY KEY (Alvazszam)
);

CREATE TABLE JarmuDolgozik (
	JarmuAzon CHARACTER VARYING,
	TeruletAzon CHARACTER VARYING,
	CONSTRAINT jarmu_kulsk FOREIGN KEY (JarmuAzon) REFERENCES Jarmu(Alvazszam),
	CONSTRAINT foldterulet_kulsk FOREIGN KEY (TeruletAzon) REFERENCES Foldterulet(HelyrajziSzam)
);
DROP TABLE Szemelyauto;
CREATE TABLE Szemelyauto (
	UtasMax INT,
	CsomagtartoMeret INT,
	CONSTRAINT szemelyauto_pk PRIMARY KEY (Alvazszam)
	
) INHERITS (Jarmu);

CREATE TABLE Teherauto (
	Kapacitas INT,
	CONSTRAINT teherauto_pk PRIMARY KEY (Alvazszam)
) INHERITS (Jarmu);

CREATE TABLE Munkagep (
	Fajta CHARACTER VARYING,
	MotorEro INT,
	CONSTRAINT munkagep_pk PRIMARY KEY (Alvazszam)
) INHERITS (Jarmu);